## Service Configuration for GCP

## Environment variables:

Define the following environment variables.

Must have:

| name | value | description | sensitive? | source |
| ---  | ---   | ---         | ---        | ---    |
| `GOOGLE_AUDIENCES` | ex `*****.apps.googleusercontent.com` | Client ID for getting access to cloud resources | yes | https://console.cloud.google.com/apis/credentials |
| `GOOGLE_CLOUD_PROJECT` | ex `project-id` | Google Cloud Project Id                                | no | - |
| `SPRING_PROFILES_ACTIVE` | ex `gcp` | Spring profile that activate default configuration for GCP environment | false | - |
| `SUBSCRIBER_PRIVATE_KEY_ID` | `********` | Private key id of DE_OPS_TESTER from notification int tests, required only for integration test| yes | output of infrastructure deployment |
| `RECORDS_CHANGE_PUBSUB_ENDPOINT` | ex `https://os-notification-dot-opendes.appspot.com/api/notification/v1/push-handlers/records-changed` | Notification external API endpoint 'records-changed'  | no | output of infrastructure deployment |

Defined in default application property file but possible to override:

| name | value | description | sensitive? | source |
| --- | --- | --- | --- | --- |
| `LOG_PREFIX` | `service` | Logging prefix | no | - |
| `SERVER_SERVLET_CONTEXPATH` | `/api/register/v1` | Register context path | no | - |
| `ENTITLEMENTS_API` | ex `http://entitlements/entitlements/v1` | Entitlements API endpoint | no | output of infrastructure deployment |
| `STORAGE_API` | ex `http://storage/api/storage/v2` | Storage API endpoint | no | output of infrastructure deployment |
| `SUBSCRIBER_SECRET` | ex`7a786376626e` | HMAC_SECRET from notification int tests in HEX , pattern(^[a-zA-Z0-9]{8,30}+$)  | yes | output of infrastructure deployment |
| `ENVIRONMENT` | ex `dev` |  Service environment config | no | - |
| `PARTITION_API` | ex `http://partition/api/partition/v1` | Partition service endpoint | no | - |
| `SERVICE_IDENTITY`| ex `osdu-gcp-sa` | Service account identity "**osdu-gcp-sa**@iam.gserviceaccount.com" | yes | https://console.cloud.google.com/apis/credentials |

These variables define service behavior, and are used to switch between `anthos` or `gcp` environments, their overriding
and usage in mixed mode was not tested. Usage of spring profiles is preferred.

| name | value | description | sensitive? | source |
| ---  | ---   | ---         | ---        | ---    |
| `PARTITION_AUTH_ENABLED` | ex `true` or `false` | Disable or enable auth token provisioning for requests to Partition service | no | - |
| `OQMDRIVER` | `rabbitmq` or `pubsub` | Oqm driver mode that defines which message broker will be used | no | - |
| `OSMDRIVER` | `datastore`| Osm driver mode that defines which KV storage will be used | no | - |
| `SERVICE_TOKEN_PROVIDER` | `GCP` or `OPENID` |Service account token provider, `GCP` means use Google service account `OPEIND` means use OpenId provider like `Keycloak` | no | - |

## Datastore configuration

There must be a namespace `DE` kind `ACTION` and `DDMS` will be created by Register service.


## Pubsub configuration

To receive messages from brokers, this solution uses the PULL-subscriber mechanism in the Notification service.
This is its cardinal difference from other implementations that use PUSH-subscribers (webhooks).
This opens a wide choice when choosing brokers.

When using PULL-subscribers, there is a need to restore Notification service subscribers for each Subscription
at the start of Notification service, as well as in the runtime,
upon registration of a new Subscription by the Register service.

To do this, a special "command" topic is involved:

At Pubsub should be created topic with name:

**name:** `register-subscriber-control`

It can be overridden by:

- through the Spring Boot property `oqm.register-subscriber-control-topic-name`
- environment variable `OQM_REGISTER_SUBSCRIBER_CONTROL_TOPIC_NAME`

See in Notification service repository in the `provider/notification-gcp/README.md` file for information
on how the Notification service at its start restores its Subscribers to all already registered Subscriptions,
and also how it listens to the "command" topic and adds its Subscribers upon registration of new Subscriptions by the Registerservice.

## Google cloud service account configuration
TBD

| Required roles |
| ---    |
| - |