/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.action.services;

import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.base.Strings;
import org.opengroup.osdu.register.action.model.Filter;
import org.opengroup.osdu.register.action.model.ParsedAction;
import org.opengroup.osdu.register.action.model.TestActionRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class TestActionService {

    @Autowired
    private ParseJsonIntoAction actionParser;

    public ParsedAction validateRegex(TestActionRequest testActionRequest) {
        // validate filter with testPayload
        if (!validateFilterWithTestPayload(testActionRequest)) {
            return ParsedAction.builder()
                    .url(testActionRequest.getAction().getUrl())
                    .errors("Filter did not match the testPayload.").build();
        }
        return this.actionParser.parse(testActionRequest.getAction().getUrl(), testActionRequest.getTestPayload());
    }

    private boolean validateFilterWithTestPayload(TestActionRequest testActionRequest) {
        Filter filter = testActionRequest.getAction().getFilter();
        JsonNode kindNode = testActionRequest.getTestPayload().get("kind");
        if (kindNode == null) {
            return false;
        }

        String kindString = kindNode.asText();
        if (Strings.isNullOrEmpty(kindString)) {
            return false;
        }

        String[] kindParams = kindString.split(":");
        if (kindParams == null || kindParams.length != 4) {
            return false;
        }

        if (!(filter.getSource().contains(kindParams[1]) || filter.getSource().contains("*"))) {
            return false;
        }

        if (!(filter.getEntityType().contains(kindParams[2]) || filter.getEntityType().contains("*"))) {
            return false;
        }

        if (!(filter.getVersion().contains(kindParams[3]) || filter.getVersion().contains("*"))) {
            return false;
        }

        return true;
    }
}
