/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.action.services;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class UrlTemplateRegexParser {

    public Map<String, String> resolveTemplateRegex(String uri) throws IllegalArgumentException{
        Map<String, String> templateAndRegex = new HashMap<>();
        int indexOfOpeningCurly = uri.indexOf('{');
        if (indexOfOpeningCurly == -1) {
            return null;
        }
        while (indexOfOpeningCurly >= 0) {
            int indexOfClosingCurly = this.indexOfCorrespondingClosingCurly(indexOfOpeningCurly, uri);
            if (indexOfClosingCurly == -1) {
                throw new IllegalArgumentException("Bad template format:　no closing curly brace found for template.");
            }
            String template = uri.substring(indexOfOpeningCurly, indexOfClosingCurly + 1);
            this.getTemplateAndRegexPair(template, templateAndRegex);
            indexOfOpeningCurly = uri.indexOf('{', indexOfClosingCurly + 1);
        }
        return templateAndRegex;
    }

    private int indexOfCorrespondingClosingCurly(int openIndex, String s) {
        LinkedList<Integer> indexOfCurlyBraces = new LinkedList<>();
        indexOfCurlyBraces.addLast(openIndex);
        for (int i = openIndex + 1; i < s.length(); i++) {
            if (s.charAt(i) == '{' && s.charAt(i - 1) != '\\') {
                indexOfCurlyBraces.add(openIndex);
            }
            if (s.charAt(i) == '}' && s.charAt(i - 1) != '\\') {
                if (indexOfCurlyBraces.size() == 1) {
                    return i;
                } else {
                    indexOfCurlyBraces.removeLast();
                }
            }
        }
        return -1;
    }

    private void getTemplateAndRegexPair(String template, Map<String, String> templateAndRegex) throws IllegalArgumentException {
        String templateContent = template.substring(1, template.length() - 1);
        if (templateContent.isEmpty()) {
            throw new IllegalArgumentException("Bad template format: empty template.");
        }
        int indexOfSeparator = templateContent.indexOf(':');
        if (indexOfSeparator == 0) {
            throw new IllegalArgumentException("Bad template format: template should contain a variable name.");
        }
        if (indexOfSeparator == -1 && this.isPureTemplateVariable(templateContent)) {
            templateAndRegex.put(template, null);
            return;
        }
        if (indexOfSeparator == -1 || !this.isLetterOrDigit(templateContent.charAt(indexOfSeparator - 1))) {
            throw new IllegalArgumentException("Bad template format: template variable and regex should be separated by colon.");
        }
        if (indexOfSeparator + 1 == templateContent.length()) {
            throw new IllegalArgumentException("Bad template format: provide regex or remove the colon.");
        }
        String regex = templateContent.substring(indexOfSeparator + 1);
        templateAndRegex.put(template, regex);
    }

    private static boolean isLetterOrDigit(char c) {
        return (c >= 'a' && c <= 'z') ||
                (c >= 'A' && c <= 'Z') ||
                (c >= '0' && c <= '9');
    }

    private boolean isPureTemplateVariable(String name) {
        Pattern varOnly = Pattern.compile("^([\\w.+]*)$");
        Matcher matcher = varOnly.matcher(name);
        return matcher.find();
    }
}
