/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.api.test;

import javax.validation.ValidationException;
import javax.validation.constraints.NotBlank;

import org.opengroup.osdu.core.common.cryptographic.ISignatureService;
import org.opengroup.osdu.core.common.cryptographic.SignatureServiceException;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.register.api.dto.ChallengeResponse;
import org.opengroup.osdu.register.provider.interfaces.subscriber.ITestSubscription;
import org.opengroup.osdu.register.utils.AppServiceConfig;
import org.opengroup.osdu.register.utils.HashingUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.annotation.RequestScope;

//used as both documentation and by integration test to validate challenge response logic
@RestController
@RequestMapping("/test")
@RequestScope
@Validated
public class SubscriberListenerApi {
    @Autowired
    private ISignatureService signatureService;
    @Autowired
    private AppServiceConfig serviceConfig;
    @Autowired
    private JaxRsDpsLog logger;
    @Autowired
    private ITestSubscription testSubscription;

    @GetMapping("challenge/{count}")
    public ChallengeResponse testCrc(@RequestParam("crc") @NotBlank String crc,
                                     @RequestParam("hmac") @NotBlank String hmac) {
        try {
            signatureService.verifyHmacSignature(hmac, this.serviceConfig.getSubscriberSecret());
            logger.debug("Signature verified and sending response");
            // Use the secret you send to the subscriber registration create request
            return new ChallengeResponse(HashingUtil.hashString(crc, this.serviceConfig.getSubscriberSecret()));
        } catch (SignatureServiceException e) {
            throw new ValidationException("Authorization signature validation Failed");
        }
    }

    @PostMapping("challenge/{count}")
    public void testPushHmac(@RequestBody Object o, @RequestParam("hmac") String hmac) {
        try {
            signatureService.verifyHmacSignature(hmac, this.serviceConfig.getSubscriberSecret());
            logger.debug("Sending acknowledgement from hmac endpoint");
            // Performing End Point Notification Acknowledgement
            try {
                testSubscription.performTestAcknowledgement();
            } catch (Exception e) {
                logger.error("An error occurred in Test Acknowledgement: " + e.toString());
            }
        } catch (SignatureServiceException e) {
            throw new ValidationException("Authorization signature validation Failed");
        }
    }
}
