/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.ddms.model.validators;

import org.opengroup.osdu.register.ddms.model.RegisteredInterface;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import static java.util.stream.Collectors.groupingBy;

public class RegisteredInterfacesValidator implements ConstraintValidator<ValidRegisteredInterfaces, Collection<RegisteredInterface>> {

    @Override
    public void initialize(ValidRegisteredInterfaces constraintAnnotation) {
        //needed by interface - we don't use
    }

    @Override
    public boolean isValid(Collection<RegisteredInterface> value, ConstraintValidatorContext context) {
        if (value.size() < 1)
            return false;
        if (value.size() > 10)
            return false;

        Map<String, List<RegisteredInterface>> ordered =
                value.stream().collect(groupingBy(RegisteredInterface::getEntityType));
        if (ordered.values().stream().anyMatch(v -> v.size() > 1))
            return false;

        return true;
    }
}
