/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.logging;

import java.util.List;
import java.util.Objects;
import lombok.RequiredArgsConstructor;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.logging.audit.AuditPayload;
import org.opengroup.osdu.core.common.logging.audit.AuditStatus;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.springframework.stereotype.Component;
import org.springframework.web.context.annotation.RequestScope;

@Component
@RequestScope
@RequiredArgsConstructor
public class AuditLogger {

    private final JaxRsDpsLog logger;
    private final DpsHeaders dpsHeaders;
    private AuditEvents auditEvents;

    private AuditEvents getAuditEvents() {
        if (Objects.isNull(this.auditEvents)) {
            this.auditEvents = new AuditEvents(this.dpsHeaders.getUserEmail());
        }
        return this.auditEvents;
    }

    public void createDmsSuccess(List<String> resources) {
        writeLog(getAuditEvents().getCreateDmsEvent(AuditStatus.SUCCESS, resources));
    }

    public void createDmsFailure(List<String> resources) {
        writeLog(getAuditEvents().getCreateDmsEvent(AuditStatus.FAILURE, resources));
    }

    public void deleteDmsSuccess(List<String> resources) {
        writeLog(getAuditEvents().getDeleteDmsEvent(AuditStatus.SUCCESS, resources));
    }

    public void deleteDmsFailure(List<String> resources) {
        writeLog(getAuditEvents().getDeleteDmsEvent(AuditStatus.FAILURE, resources));
    }

    public void createSubscriptionSuccess(List<String> resources) {
        writeLog(getAuditEvents().getCreateSubscriptionEvent(AuditStatus.SUCCESS, resources));
    }

    public void createSubscriptionFailure(List<String> resources) {
        writeLog(getAuditEvents().getCreateSubscriptionEvent(AuditStatus.FAILURE, resources));
    }

    public void deleteSubscriptionSuccess(List<String> resources) {
        writeLog(getAuditEvents().getDeleteSubscriptionEvent(AuditStatus.SUCCESS, resources));
    }

    public void deleteSubscriptionFailure(List<String> resources) {
        writeLog(getAuditEvents().getDeleteSubscriptionEvent(AuditStatus.FAILURE, resources));
    }

    public void updateSubscriptionSecretSuccess(List<String> resources) {
        writeLog(getAuditEvents().getUpdateSubscriptionSecretEvent(AuditStatus.SUCCESS, resources));
    }

    public void updateSubscriptionSecretFailure(List<String> resources) {
        writeLog(getAuditEvents().getUpdateSubscriptionSecretEvent(AuditStatus.FAILURE, resources));
    }

    public void readActionSuccess(List<String> resources) {
        writeLog(getAuditEvents().getReadActionEvent(AuditStatus.SUCCESS, resources));
    }

    public void readActionFailure(List<String> resources) {
        writeLog(getAuditEvents().getReadActionEvent(AuditStatus.FAILURE, resources));
    }

    public void readMultipleActionsSuccess(List<String> resources) {
        writeLog(getAuditEvents().getReadMultipleActionsEvent(AuditStatus.SUCCESS, resources));
    }

    public void readMultipleActionsFailure(List<String> resources) {
        writeLog(getAuditEvents().getReadMultipleActionsEvent(AuditStatus.FAILURE, resources));
    }

    public void createActionSuccess(List<String> resources) {
        writeLog(getAuditEvents().getCreateActionEvent(AuditStatus.SUCCESS, resources));
    }

    public void createActionFailure(List<String> resources) {
        writeLog(getAuditEvents().getCreateActionEvent(AuditStatus.FAILURE, resources));
    }

    public void deleteActionSuccess(List<String> resources) {
        writeLog(getAuditEvents().getDeleteActionEvent(AuditStatus.SUCCESS, resources));
    }

    public void deleteActionFailure(List<String> resources) {
        writeLog(getAuditEvents().getDeleteActionEvent(AuditStatus.FAILURE, resources));
    }

    public void validateActionSuccess(List<String> resources) {
        writeLog(getAuditEvents().getValidateActionEvent(AuditStatus.SUCCESS, resources));
    }

    public void validateActionFailure(List<String> resources) {
        writeLog(getAuditEvents().getValidateActionEvent(AuditStatus.FAILURE, resources));
    }

    public void readSubscriptionSuccess(List<String> resources) {
        writeLog(getAuditEvents().getReadSubscriptionEvent(AuditStatus.SUCCESS, resources));
    }

    public void readSubscriptionFailure(List<String> resources) {
        writeLog(getAuditEvents().getReadSubscriptionEvent(AuditStatus.FAILURE, resources));
    }

    private void writeLog(AuditPayload log) {
        logger.audit(log);
    }
}
