/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.subscriber.services;

import java.util.Collections;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.logging.AuditLogger;
import org.opengroup.osdu.register.provider.interfaces.subscriber.ISubscriptionRepository;
import org.opengroup.osdu.register.subscriber.model.Subscription;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Base64;
import java.util.UUID;

@Service
public class CreateSubscription {

    @Autowired
    private ISubscriptionRepository subscriptionRepository;
    @Autowired
    private ChallengeResponseCheck challengeResponseCheck;
    @Autowired
    private AvailableTopics topics;
    @Autowired
    private DpsHeaders dpsHeaders;
    @Autowired
    private AuditLogger auditLogger;

    public Subscription run(Subscription input) throws Exception {
        if (!topics.exists(input.getTopic())) {
            throw new AppException(400, "Bad Request",
                    String.format("Given topic name does not match any available topics. Given: %s Available: %s",
                            input.getTopic(), topics.listNames()));
        }

        if (challengeResponseCheck.runCreate(input)) {
            input.setCreatedBy(dpsHeaders.getUserEmail());
            input.setNotificationId("de-" + UUID.randomUUID().toString());
            String id = Base64.getEncoder().encodeToString((input.getTopic() + input.getPushEndpoint()).getBytes());
            input.setId(id);
            input = subscriptionRepository.create(input);
            this.auditLogger.createSubscriptionSuccess(Collections.singletonList(input.toString()));
        } else {
            throw new AppException(400, "Bad Request", "Failed challenge response check to GET " + input.getPushEndpoint());
        }
        return input;
    }
}
