/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.utils;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.annotation.RequestScope;

import javax.servlet.http.HttpServletRequest;

@RequestScope
@Component
public class ServiceRequestInfo {

    @Autowired
    private HttpServletRequest httpRequest;
    @Autowired
    private AppServiceConfig serviceConfig;

    private static final String EXPECTED_HEADER_VALUE = "true";

    public boolean isHttps() {
        return getUri().startsWith("https") || "https".equalsIgnoreCase(this.httpRequest.getHeader("x-forwarded-proto"));
    }

    public String getUri() {
        StringBuilder requestURL = new StringBuilder(httpRequest.getRequestURL().toString());
        String queryString = httpRequest.getQueryString();

        if (queryString == null) {
            return requestURL.toString();
        } else {
            return requestURL.append('?').append(queryString).toString();
        }
    }

    public String getUserIp() {
        return httpRequest.getRemoteAddr();
    }

    public boolean isCronRequest() {
        String appEngineCronHeader = httpRequest.getHeader("X-Appengine-Cron");
        return EXPECTED_HEADER_VALUE.equalsIgnoreCase(appEngineCronHeader) &&
                this.serviceConfig.getCronJobExpectedIp().equalsIgnoreCase(getUserIp());

    }
}
