/* Copyright © 2020 Amazon Web Services
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.subscriber;

import com.google.common.base.Strings;
import com.google.gson.Gson;
import com.sun.jersey.api.client.ClientResponse;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.opengroup.osdu.register.model.Subscriber;
import org.opengroup.osdu.register.util.AwsTestUtils;
import org.opengroup.osdu.register.util.Config;
import org.opengroup.osdu.register.util.RestDescriptor;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

public class TestCreateSubscriberApi extends CreateSubscriberApiTest {

    @Before
    @Override
    public void setup() throws Exception {
        this.testUtils = new AwsTestUtils();
        super.setup();
    }

    @After
    @Override
    public void tearDown() {
        this.testUtils = null;
    }

    @Override
    protected void validate20XResponse(ClientResponse response, RestDescriptor descriptor) {
        //String url = Config.Instance().PushUrl + "api/register/v1/test/challenge/1";
        String url = "";
        String pushPath = System.getProperty("REGISTER_CUSTOM_PUSH_URL1", System.getenv("REGISTER_CUSTOM_PUSH_URL1"));
        if( Strings.isNullOrEmpty(pushPath)) {
            url = Config.Instance().PushUrl + "api/register/v1/test/challenge/1";
        }
        else
        {
            url = pushPath;
        }
        String body = response.getEntity(String.class);
        Subscriber subscriber = new Gson().fromJson(body, Subscriber.class);
        String createdBy = System.getProperty("AWS_COGNITO_AUTH_PARAMS_USER", System.getenv("AWS_COGNITO_AUTH_PARAMS_USER"));

        assertEquals("My test listener.", subscriber.description);
        assertEquals(createdBy, subscriber.createdBy);
        assertEquals("My listener", subscriber.name);
        assertEquals("records-changed", subscriber.topic);
        assertFalse(Strings.isNullOrEmpty(subscriber.notificationId));
        assertEquals(url, subscriber.pushEndpoint);
    }


    //overriding test to return 401 for AWS Istio config
    @Test
    @Override
    public void should_return400_when_makingHttpRequestWithoutToken() throws Exception {
        ClientResponse response = descriptor.run(getId(), "");
        assertEquals(error(response.getEntity(String.class)), 401, response.getStatus());
    }
}
